package be.iminds.ilabt.jfed.testing.tests.highlevel.config;

import be.iminds.ilabt.jfed.experiment.setup.config.WaitForReady;
import be.iminds.ilabt.jfed.testing.base.ApiTestConfigProperty;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;

import javax.annotation.Nullable;
import java.util.Objects;

public class WaitForReadyInTest extends WaitForReady {
    private final boolean skipFinalManifestCheck;
    private final boolean requireNodesInManifest;

    public WaitForReadyInTest() {
        super();
        this.skipFinalManifestCheck = false;
        this.requireNodesInManifest = true;
    }

    @JsonCreator
    public WaitForReadyInTest(@Nullable @JsonProperty("maxTimeMin") Long maxTimeMin,
                        @Nullable @JsonProperty("skipFinalManifestCheck") Boolean skipFinalManifestCheck,
                        @Nullable @JsonProperty("requireNodesInManifest") Boolean requireNodesInManifest,
                        @Nullable @JsonProperty("extraWaitTimeSeconds") Long extraWaitTimeSeconds,
                        @Nullable @JsonProperty("enabled") Boolean enabled) {
        super(maxTimeMin, extraWaitTimeSeconds, enabled);
        this.skipFinalManifestCheck = skipFinalManifestCheck == null ? new WaitForReadyInTest().getSkipFinalManifestCheck() : skipFinalManifestCheck;
        this.requireNodesInManifest = requireNodesInManifest == null ? new WaitForReadyInTest().getRequireNodesInManifest() : requireNodesInManifest;
    }

    @JsonIgnore
    public boolean isValid() {
        if (!enabled)
            return true;

        if (maxTimeMin < 0)
            throw new IllegalArgumentException("Config Error: waitForReady.maxTimeMin < 0 makes no sense");

        return true;
    }

    @JsonProperty
    @ApiTestConfigProperty(description="Skip the detailed manifest test in the \"checkManifest\" step? If skipped, very basic testing of the manifest is still done.")
    public boolean getSkipFinalManifestCheck() {
        return skipFinalManifestCheck;
    }


    @JsonProperty
    public long getMaxTimeMin() {
        return maxTimeMin;
    }

    @JsonProperty
    public boolean getRequireNodesInManifest() {
        return requireNodesInManifest;
    }

    @JsonProperty
    public long getExtraWaitTimeSeconds() {
        return extraWaitTimeSeconds;
    }

    @JsonProperty
    public boolean isEnabled() {
        return enabled;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        WaitForReadyInTest that = (WaitForReadyInTest) o;
        return skipFinalManifestCheck == that.skipFinalManifestCheck &&
                requireNodesInManifest == that.requireNodesInManifest;
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), skipFinalManifestCheck, requireNodesInManifest);
    }
}
